{% extends "layout.html" %}
{% set page_links = [
  (pathto('@rss/' + sourcename), 'application/rss+xml', 'Page Comments'),
] %}
{% block body %}
  {% if oldurl %}
  <div class="docwarning">
    {% trans %}<strong>Note:</strong> You requested an out-of-date URL from this server. We've tried to redirect you to the new location of this page, but it may not be the right one.{% endtrans %}
  </div>
  {% endif %}
  {{ body }}
{% endblock %}
